@@ -22,9 +22,9 @@
 #include "config.h"
 
 #include "pango-break.h"
-#include "pango-engine-private.h"
 #include "pango-script-private.h"
 #include "pango-emoji-private.h"
+#include "pango-break-table.h"
 #include "pango-impl-utils.h"
 #include <string.h>
 
@@ -124,7 +124,8 @@ static const CharJamoProps HangulJamoProps[] = {
 #define GREEK(wc) (((wc) >= 0x0370 && (wc) <= 0x3FF) || ((wc) >= 0x1F00 && (wc) <= 0x1FFF))
 #define KANA(wc) ((wc) >= 0x3040 && (wc) <= 0x30FF)
 #define HANGUL(wc) ((wc) >= 0xAC00 && (wc) <= 0xD7A3)
-#define BACKSPACE_DELETES_CHARACTER(wc) (!LATIN (wc) && !CYRILLIC (wc) && !GREEK (wc) && !KANA(wc) && !HANGUL(wc))
+#define EMOJI(wc) (_pango_Is_Emoji_Base_Character (wc))
+#define BACKSPACE_DELETES_CHARACTER(wc) (!LATIN (wc) && !CYRILLIC (wc) && !GREEK (wc) && !KANA (wc) && !HANGUL (wc) && !EMOJI (wc))
 
 /* Previously "123foo" was two words. But in UAX 29 of Unicode, 
  * we know don't break words between consecutive letters and numbers
@@ -139,19 +140,17 @@ typedef enum
 
 /**
  * pango_default_break:
- * @text: text to break
+ * @text: text to break. Must be valid UTF-8
  * @length: length of text in bytes (may be -1 if @text is nul-terminated)
  * @analysis: (nullable): a #PangoAnalysis for the @text
  * @attrs: logical attributes to fill in
  * @attrs_len: size of the array passed as @attrs
  *
- * This is the default break algorithm, used if no language
- * engine overrides it. Normally you should use pango_break()
- * instead. Unlike pango_break(),
- * @analysis can be %NULL, but only do that if you know what
- * you're doing. If you need an analysis to pass to pango_break(),
- * you need to pango_itemize().  In most cases however you should
- * simply use pango_get_log_attrs().
+ * This is the default break algorithm. It applies Unicode
+ * rules without language-specific tailoring, therefore
+ * the @analyis argument is unused and can be %NULL.
+ *
+ * See pango_tailor_break() for language-specific breaks.
  **/
 void
 pango_default_break (const gchar   *text,
@@ -523,6 +522,11 @@ pango_default_break (const gchar   *text,
 	if (is_grapheme_boundary)
           {
 	    attrs[i].backspace_deletes_character = BACKSPACE_DELETES_CHARACTER (base_character);
+
+	    /* Dependent Vowels for Indic language */
+	    if (_pango_is_Virama (prev_wc) ||
+		_pango_is_Vowel_Dependent (prev_wc))
+	      attrs[i].backspace_deletes_character = TRUE;
           }
 	else
 	  attrs[i].backspace_deletes_character = FALSE;
@@ -539,7 +543,7 @@ pango_default_break (const gchar   *text,
 	    PangoScript script;
 	    WordBreakType WB_type;
 
-	    script = g_unichar_get_script (wc);
+	    script = (PangoScript)g_unichar_get_script (wc);
 
 	    /* Find the WordBreakType of wc */
 	    WB_type = WB_Other;
@@ -845,69 +849,7 @@ pango_default_break (const gchar   *text,
 		      wc == 0xFF64)
 		    SB_type = SB_SContinue;
 
-		  if (wc == 0x0021 ||
-		      wc == 0x003F ||
-		      wc == 0x0589 ||
-		      wc == 0x061F ||
-		      wc == 0x06D4 ||
-		      (wc >= 0x0700 && wc <= 0x0702) ||
-		      wc == 0x07F9 ||
-		      (wc >= 0x0964 && wc <= 0x0965) ||
-		      (wc >= 0x104A && wc <= 0x104B) ||
-		      wc == 0x1362 ||
-		      (wc >= 0x1367 && wc <= 0x1368) ||
-		      wc == 0x166E ||
-		      (wc >= 0x1735 && wc <= 0x1736) ||
-		      wc == 0x1803 ||
-		      wc == 0x1809 ||
-		      (wc >= 0x1944 && wc <= 0x1945) ||
-		      (wc >= 0x1AA8 && wc <= 0x1AAB) ||
-		      (wc >= 0x1B5A && wc <= 0x1B5B) ||
-		      (wc >= 0x1B5E && wc <= 0x1B5F) ||
-		      (wc >= 0x1C3B && wc <= 0x1C3C) ||
-		      (wc >= 0x1C7E && wc <= 0x1C7F) ||
-		      (wc >= 0x203C && wc <= 0x203D) ||
-		      (wc >= 0x2047 && wc <= 0x2049) ||
-		      wc == 0x2E2E ||
-		      wc == 0x2E3C ||
-		      wc == 0x3002 ||
-		      wc == 0xA4FF ||
-		      (wc >= 0xA60E && wc <= 0xA60F) ||
-		      wc == 0xA6F3 ||
-		      wc == 0xA6F7 ||
-		      (wc >= 0xA876 && wc <= 0xA877) ||
-		      (wc >= 0xA8CE && wc <= 0xA8CF) ||
-		      wc == 0xA92F ||
-		      (wc >= 0xA9C8 && wc <= 0xA9C9) ||
-		      (wc >= 0xAA5D && wc <= 0xAA5F) ||
-		      (wc >= 0xAAF0 && wc <= 0xAAF1) ||
-		      wc == 0xABEB ||
-		      (wc >= 0xFE56 && wc <= 0xFE57) ||
-		      wc == 0xFF01 ||
-		      wc == 0xFF1F ||
-		      wc == 0xFF61 ||
-		      (wc >= 0x10A56 && wc <= 0x10A57) ||
-		      (wc >= 0x11047 && wc <= 0x11048) ||
-		      (wc >= 0x110BE && wc <= 0x110C1) ||
-		      (wc >= 0x11141 && wc <= 0x11143) ||
-		      (wc >= 0x111C5 && wc <= 0x111C6) ||
-		      wc == 0x111CD ||
-		      (wc >= 0x111DE && wc <= 0x111DF) ||
-		      (wc >= 0x11238 && wc <= 0x11239) ||
-		      (wc >= 0x1123B && wc <= 0x1123C) ||
-		      wc == 0x112A9 ||
-		      (wc >= 0x1144B && wc <= 0x1144C) ||
-		      (wc >= 0x115C2 && wc <= 0x115C3) ||
-		      (wc >= 0x115C9 && wc <= 0x115D7) ||
-		      (wc >= 0x11641 && wc <= 0x11642) ||
-		      (wc >= 0x1173C && wc <= 0x1173E) ||
-		      (wc >= 0x11C41 && wc <= 0x11C42) ||
-		      (wc >= 0x16A6E && wc <= 0x16A6F) ||
-		      wc == 0x16AF5 ||
-		      (wc >= 0x16B37 && wc <= 0x16B38) ||
-		      wc == 0x16B44 ||
-		      wc == 0x1BC9F ||
-		      wc == 0x1DA88)
+		  if (_pango_is_STerm(wc))
 		    SB_type = SB_STerm;
 
 		  break;
@@ -1629,28 +1571,46 @@ pango_default_break (const gchar   *text,
 }
 
 static gboolean
-tailor_break (const gchar   *text,
-	     gint           length,
-	     PangoAnalysis *analysis,
-	     PangoLogAttr  *attrs,
-	     int            attrs_len)
-{
-  if (analysis->lang_engine && PANGO_ENGINE_LANG_GET_CLASS (analysis->lang_engine)->script_break)
-    {
-      if (length < 0)
-	length = strlen (text);
-      else if (text == NULL)
-	text = "";
+break_script (const char          *item_text,
+	      unsigned int         item_length,
+	      const PangoAnalysis *analysis,
+	      PangoLogAttr        *attrs,
+	      int                  attrs_len);
 
-      PANGO_ENGINE_LANG_GET_CLASS (analysis->lang_engine)->script_break (analysis->lang_engine, text, length, analysis, attrs, attrs_len);
-      return TRUE;
-    }
-  return FALSE;
+static gboolean
+break_attrs (const char   *text,
+	     int           length,
+             GSList       *attributes,
+             int           item_offset,
+             PangoLogAttr *attrs,
+             int           attrs_len);
+
+static gboolean
+tailor_break (const char    *text,
+	      int            length,
+	      PangoAnalysis *analysis,
+              int            item_offset,
+	      PangoLogAttr  *attrs,
+	      int            attrs_len)
+{
+  gboolean res;
+
+  if (length < 0)
+    length = strlen (text);
+  else if (text == NULL)
+    text = "";
+
+  res = break_script (text, length, analysis, attrs, attrs_len);
+
+  if (item_offset >= 0)
+    res |= break_attrs (text, length, analysis->extra_attrs, item_offset, attrs, attrs_len);
+
+  return res;
 }
 
 /**
  * pango_break:
- * @text:      the text to process
+ * @text:      the text to process. Must be valid UTF-8
  * @length:    length of @text in bytes (may be -1 if @text is nul-terminated)
  * @analysis:  #PangoAnalysis structure from pango_itemize()
  * @attrs:     (array length=attrs_len): an array to store character
@@ -1658,8 +1618,10 @@ tailor_break (const gchar   *text,
  * @attrs_len: size of the array passed as @attrs
  *
  * Determines possible line, word, and character breaks
- * for a string of Unicode text with a single analysis.  For most
- * purposes you may want to use pango_get_log_attrs().
+ * for a string of Unicode text with a single analysis.
+ * For most purposes you may want to use pango_get_log_attrs().
+ *
+ * Deprecated: 1.44: Use pango_default_break() and pango_tailor_break()
  */
 void
 pango_break (const gchar   *text,
@@ -1672,7 +1634,7 @@ pango_break (const gchar   *text,
   g_return_if_fail (attrs != NULL);
 
   pango_default_break (text, length, analysis, attrs, attrs_len);
-  tailor_break        (text, length, analysis, attrs, attrs_len);
+  tailor_break        (text, length, analysis, -1, attrs, attrs_len);
 }
 
 /**
@@ -1775,6 +1737,51 @@ pango_find_paragraph_boundary (const gchar *text,
     *next_paragraph_start = start - text;
 }
 
+/**
+ * pango_tailor_break:
+ * @text: text to process. Must be valid UTF-8
+ * @length: length in bytes of @text
+ * @analysis:  #PangoAnalysis structure from pango_itemize() for @text
+ * @offset: Byte offset of @text from the beginning of the
+ *     paragraph, or -1 to ignore attributes from @analysis
+ * @log_attrs: (array length=attrs_len): array with one #PangoLogAttr
+ *   per character in @text, plus one extra, to be filled in
+ * @attrs_len: length of @log_attrs array
+ *
+ * Apply language-specific tailoring to the breaks in
+ * @log_attrs, which are assumed to have been produced
+ * by pango_default_break().
+ *
+ * If @offset is not -1, it is used to apply attributes
+ * from @analysis that are relevant to line breaking.
+ *
+ * Since: 1.44
+ */
+void
+pango_tailor_break (const char    *text,
+                    int            length,
+                    PangoAnalysis *analysis,
+                    int            offset,
+                    PangoLogAttr  *log_attrs,
+                    int            log_attrs_len)
+{
+  PangoLogAttr *start = log_attrs;
+  PangoLogAttr attr_before = *start;
+
+  if (tailor_break (text, length, analysis, offset, log_attrs, log_attrs_len))
+    {
+      /* if tailored, we enforce some of the attrs from before
+       * tailoring at the boundary
+       */
+
+     start->backspace_deletes_character  = attr_before.backspace_deletes_character;
+
+     start->is_line_break      |= attr_before.is_line_break;
+     start->is_mandatory_break |= attr_before.is_mandatory_break;
+     start->is_cursor_position |= attr_before.is_cursor_position;
+    }
+}
+
 static int
 tailor_segment (const char      *range_start,
 		const char      *range_end,
@@ -1784,33 +1791,22 @@ tailor_segment (const char      *range_start,
 {
   int chars_in_range;
   PangoLogAttr *start = log_attrs + chars_broken;
-  PangoLogAttr attr_before = *start;
 
   chars_in_range = pango_utf8_strlen (range_start, range_end - range_start);
 
-  if (tailor_break (range_start,
-		    range_end - range_start,
-		    analysis,
-		    start,
-		    chars_in_range + 1))
-    {
-      /* if tailored, we enforce some of the attrs from before tailoring at
-       * the boundary
-       */
-
-     start->backspace_deletes_character  = attr_before.backspace_deletes_character;
-
-     start->is_line_break      |= attr_before.is_line_break;
-     start->is_mandatory_break |= attr_before.is_mandatory_break;
-     start->is_cursor_position |= attr_before.is_cursor_position;
-    }
+  pango_tailor_break (range_start,
+                      range_end - range_start,
+                      analysis,
+                      -1,
+                      start,
+                      chars_in_range + 1);
 
   return chars_in_range;
 }
 
 /**
  * pango_get_log_attrs:
- * @text: text to process
+ * @text: text to process. Must be valid UTF-8
  * @length: length in bytes of @text
  * @level: embedding level, or -1 if unknown
  * @language: language tag
@@ -1842,7 +1838,6 @@ pango_get_log_attrs (const char    *text,
   g_return_if_fail (log_attrs != NULL);
 
   analysis.level = level;
-  analysis.lang_engine = _pango_get_language_engine ();
 
   pango_default_break (text, length, &analysis, log_attrs, attrs_len);
 
@@ -1872,7 +1867,7 @@ pango_get_log_attrs (const char    *text,
 #include "break-indic.c"
 #include "break-thai.c"
 
-static void
+static gboolean
 break_script (const char          *item_text,
 	      unsigned int         item_length,
 	      const PangoAnalysis *analysis,
@@ -1901,43 +1896,66 @@ break_script (const char          *item_text,
     case PANGO_SCRIPT_THAI:
       break_thai (item_text, item_length, analysis, attrs, attrs_len);
       break;
+    default:
+      return FALSE;
     }
+
+  return TRUE;
 }
 
+static gboolean
+break_attrs (const char   *text,
+             int           length,
+             GSList       *attributes,
+             int           offset,
+             PangoLogAttr *log_attrs,
+             int           log_attrs_len)
+{
+  PangoAttrList *list;
+  PangoAttrIterator *iter;
+  GSList *l;
 
-/* Wrap language breaker in PangoEngineLang to pass it through old API,
- * from times when there were modules and engines. */
-typedef PangoEngineLang      PangoLanguageEngine;
-typedef PangoEngineLangClass PangoLanguageEngineClass;
-static GType pango_language_engine_get_type (void) G_GNUC_CONST;
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-G_DEFINE_TYPE (PangoLanguageEngine, pango_language_engine, PANGO_TYPE_ENGINE_LANG);
-G_GNUC_END_IGNORE_DEPRECATIONS
-static void
-_pango_language_engine_break (PangoEngineLang *engine G_GNUC_UNUSED,
-			      const char      *item_text,
-			      int              item_length,
-			      PangoAnalysis   *analysis,
-			      PangoLogAttr    *attrs,
-			      int              attrs_len)
-{
-  break_script (item_text, item_length, analysis, attrs, attrs_len);
-}
-static void
-pango_language_engine_class_init (PangoEngineLangClass *class)
-{
-  class->script_break = _pango_language_engine_break;
-}
-static void
-pango_language_engine_init (PangoEngineLang *object)
-{
-}
+  list = pango_attr_list_new ();
+  for (l = attributes; l; l = l->next)
+    {
+      PangoAttribute *attr = l->data;
 
-PangoEngineLang *
-_pango_get_language_engine (void)
-{
-  static PangoEngineLang *engine;
-  if (g_once_init_enter (&engine))
-    g_once_init_leave (&engine, g_object_new (pango_language_engine_get_type(), NULL));
-  return engine;
+      if (attr->klass->type == PANGO_ATTR_ALLOW_BREAKS)
+        pango_attr_list_insert (list, (PangoAttribute*)l->data);
+    }
+
+  iter = pango_attr_list_get_iterator (list);
+  do {
+    PangoAttribute *attr;
+
+    attr = pango_attr_iterator_get (iter, PANGO_ATTR_ALLOW_BREAKS);
+    if (attr && ((PangoAttrInt*)attr)->value == 0)
+      {
+        int start, end;
+        int start_pos, end_pos;
+        int pos;
+
+        pango_attr_iterator_range (iter, &start, &end);
+        if (start < offset)
+          start_pos = 0;
+        else
+          start_pos = g_utf8_pointer_to_offset (text, text + start - offset);
+        if (end >= offset + length)
+          end_pos = log_attrs_len;
+        else
+          end_pos = g_utf8_pointer_to_offset (text, text + end - offset);
+
+        for (pos = start_pos + 1; pos < end_pos; pos++)
+          {
+            log_attrs[pos].is_mandatory_break = FALSE;
+            log_attrs[pos].is_line_break = FALSE;
+            log_attrs[pos].is_char_break = FALSE;
+          }
+      }
+  } while (pango_attr_iterator_next (iter));
+
+  pango_attr_iterator_destroy (iter);
+  pango_attr_list_unref (list);
+
+  return TRUE;
 }
